/* @flow */'use strict';Object.defineProperty(exports, '__esModule', { value: true });var _createClass = (function () {function defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ('value' in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}return function (Constructor, protoProps, staticProps) {if (protoProps) defineProperties(Constructor.prototype, protoProps);if (staticProps) defineProperties(Constructor, staticProps);return Constructor;};})();function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { 'default': obj };}function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError('Cannot call a class as a function');}}var _compilerCompileResult = require(

'./compiler/CompileResult');var _compilerCompileResult2 = _interopRequireDefault(_compilerCompileResult);var _compilerCompiler = require(
'./compiler/Compiler');var _compilerCompiler2 = _interopRequireDefault(_compilerCompiler);var _Config = require(
'./Config');var _Config2 = _interopRequireDefault(_Config);var _routerRouter = require(
'./router/Router');var _routerRouter2 = _interopRequireDefault(_routerRouter);var _EntryPoint = require(
'./EntryPoint');var _EntryPoint2 = _interopRequireDefault(_EntryPoint);var _httpServerHttpRequest = require(
'./http/ServerHttpRequest');var _httpServerHttpRequest2 = _interopRequireDefault(_httpServerHttpRequest);

var path = require('path');
var http = require('http');

/**
 * Orchestrates all the things that makes the application server run:
 *
 * 1. Gets the list of entry points from Router implementation.
 * 2. Compiles the code with the given Compiler implementation.
 * 3. Create and run the server.
 * 4. Handles the requests.
 *
 * In handling the requests:
 *
 * 1. If asset request, pass it to the AssetServer implementation.
 * 2. If not, use Router to determine which page to run.
 * 2. Pass request and response object to Page.
 *
 * In error handling:
 *
 * TODO: Think about error handling!
 * TODO: Make every process async with Promise!
 *
 * This object is stateless, should not store ANY request-specific states.
 */var 
Application = (function () {











  function Application(
  componentRegister, router /*: Router*/, 
  compiler /*: Compiler*/, frameworkConfig /*: Object*/, serverConfig /*: Object*/, 
  clientConfig /*: Object*/) 
  {_classCallCheck(this, Application);
    var defaultFrameworkConfig = { 
      port: 8000, 
      minifyJs: false, 
      clientReplace: {}, 
      clientResolve: [], 
      absoluteComponentsDir: [] };

    this._componentRegister = componentRegister;
    this._compiler = compiler;
    this._assetServer = this._compiler.getAssetServer();
    this._frameworkConfig = Object.assign(defaultFrameworkConfig, frameworkConfig);
    this._serverConfig = serverConfig;
    this._clientConfig = clientConfig;
    this._router = router;
    this._pages = {};
    this._entryPoints = [];

    var i, pageCmpt, page, pageClass, pageComponents = componentRegister.getPages();
    for (i in pageComponents) {
      if (!pageComponents.hasOwnProperty(i)) continue;
      pageCmpt = pageComponents[i];

      // Create entry point.
      this._entryPoints.push(new _EntryPoint2['default'](pageCmpt.name, pageCmpt.absDir));

      // Instantiate page.
      pageClass = pageCmpt.clazz;
      page = new pageClass();
      page.init(serverConfig);
      this._pages[pageCmpt.name] = page;}}



  /**
   * Compiles and then create an http server that handles requests.
   *
   * TODO: multiple ports, port to listen to in configuration.
   */_createClass(Application, [{ key: 'run', value: 
    function run() {
      this.compile();}


    /**
     * Runs runtime compilation. This generates compilation result, which is the
     * list of CSS and JS dependencies of each page, so that server can render
     * the HTML page.
     *
     * TODO: Separate compile, listen, and handle - don't tie them together for user flexibility.
     */ }, { key: 'compile', value: 
    function compile() {
      this._compileResult = this._compiler.run(this._entryPoints, (function (compileResult) {
        this._compileResult = compileResult;
        this.createServer();}).
      bind(this));} }, { key: 'createServer', value: 


    function createServer() {
      var self = this;
      http.createServer(function (request, response) {
        try {
          var serverHttpRequest = new _httpServerHttpRequest2['default'](request);
          self.handle(serverHttpRequest, response);} 
        catch (e) {
          self.handleError(e, request, response);}}).

      listen(this._frameworkConfig.port);
      // TODO: Use modifiable logging class!
      console.log('Server listening at port: ' + this._frameworkConfig.port);} }, { key: 'handle', value: 


    function handle(httpRequest /*: ServerHttpRequest*/, response /*: any*/) {
      // Check if it's an asset request. If asset request pass through asset
      // container immediately.
      var handledByAssetServer = this._assetServer.handle(httpRequest, response);
      if (handledByAssetServer) {
        return;}


      // Else route through router to get the page instance.
      var routeResult = this._router.route(httpRequest);
      if (routeResult == null) {
        throw new Error('404');}


      var page = this._pages[routeResult.pageName];
      if (page == null) {
        throw new Error('404');}


      var renderResult = page.render(httpRequest, routeResult.routeArgs);
      var pageDep = this._compileResult.pages[routeResult.pageName];
      if (!pageDep) {
        throw new Error('Unable to render page server side, dependencies unknown for entry point: ' + routeResult.componentName);}


      var htmlResult = this._compiler.assembleHtml(
      routeResult.pageName, routeResult.routeArgs, pageDep, renderResult, 
      this._clientConfig, httpRequest.isSecure());


      // TODO: Render result to be able to set status code, etc. Should be similar to HttpResult.
      response.writeHead(200, { 'Content-Type': 'text/html' });
      response.end(htmlResult);} }, { key: 'handleError', value: 


    function handleError(error, httpRequest, response) {
      // TODO: Error handling implementation!
      response.writeHead(200, { 'Content-Type': 'text/html' });
      if (error instanceof Error) {
        response.end('<pre>' + error.stack + '</pre>');
        return;}

      response.end(error + '');} }]);return Application;})();exports['default'] = Application;module.exports = exports['default'];
//# sourceMappingURL=Application.js.map